__doc__ = """
TestResults
"""
__version__ = '0.1.0'

import Globals, Acquisition, AccessControl.Role, OFS.SimpleItem, OFS.PropertyManager
import Products, DateTime
import string, StringIO
import unittest

class StringAndHttpIO (StringIO.StringIO):
	def __init__ (self, response):
		StringIO.StringIO.__init__ (self)
		self._response = response

	def write (self, s):
		StringIO.StringIO.write (self, s)
		self._response.write (s)
	
	def flush (self):
		StringIO.StringIO.flush (self)
		# this is a hack and may be ugly, but I don't feel quite ready to
		# maintain a new TestTesult subclass yet just for the \n
		self._response.write ('\n')


_tres_deprecated_attrs = ('_result')
_tres_prototype = {
		'title': '', '_success': 0, '_messages': '(fake test)',
	}

class TestResults (
	OFS.SimpleItem.SimpleItem,
	#OFS.PropertyManager.PropertyManager,
	):
	"""TestResults class
	"""

	#################################################
	### Zope class attributes

	name = 'TestResults'
	id = 'TestResults'
	meta_type = 'ZUnit TestResults'
	#icon = 'misc_/ZUnit/TestResults.gif'

	_properties = (
			{'id': 'title', 'type': 'string', 'mode': 'w'},
			{'id': 'suite', 'type': 'string', 'mode': 'w'},
		)

	manage_options = (
			{'label':'View', 'action':''},
		)


	__ac_permissions__ = (
			('View management screens',
			 ('manage_tabs',)),
			('View',
			 ('', 'index_html', '__call__',)),
			('Know the secrets of life',
			 ('manage_dict', 'manage_foo',)),
			)


	#################################################
	## Instance constructors

	def __init__ (self, id, suiteCreator, ctx):
		"Initialize the class instance"
		self.id = id
		for name, value in _tres_prototype.items ():
			if not hasattr (self, name):
				setattr (self, name, value)
		suite = unittest.createTestInstance (suiteCreator)
		if hasattr (suite, 'setZopeContext'):
			suite.setZopeContext (ctx[id + '.data'])
		try:
			stream = StringAndHttpIO (ctx.REQUEST.RESPONSE)
		except AttributeError:
			stream = StringIO.StringIO()
		self._success = unittest.TextTestRunner (stream).run (suite).wasSuccessful()
		self._messages = stream.getvalue()

	def __setstate__ (self, state):
		"Initialize when loading from the ZODB"
		Globals.Persistent.__setstate__ (self, state)
		for name, value in _tres_prototype.items ():
			if not hasattr (self, name):
				setattr (self, name, value)
		for name in _tres_deprecated_attrs:
			if hasattr (self, name):
				delattr (self, name)


	#################################################
	## Misc ObjectManager stuff
	all_meta_types = ()


	#################################################
	## Management interface
	# None yet

	#################################################
	## Externally accessible, but not from the manage
	## interface
	
	def index_html (self, REQUEST, **kw):
		"Render the data into a webpage"
		return apply (self.__str__, (self, REQUEST), kw)


	#################################################
	## Externally accessible from Python/DTML only
	
	def __str__ (self, context=None, REQUEST=None, **kw):
		"Render the data"
		return self._messages

	def __call__ (self, context=None, REQUEST=None, **kw):
		"Render the data given a context (?)"
		# DocumentTemplate calls us with (None, context)
		if context is None and hasattr (REQUEST, 'has_key') and REQUEST.has_key ('REQUEST'):
			context = REQUEST
			REQUEST = context['REQUEST']
		# FIXME: this will lose our properties when context != self...
		return apply (self.__str__, (context, REQUEST), kw)


	#################################################
	## Methods for internal consumption
	


	#################################################
	## Debugging stuff, in the "what not even God
	## should do" category
	def manage_dict (self, REQUEST=None):
		"Debugging method - meaning is obvious"
		return `self.__dict__`
