__doc__ = """
TestRunner
"""
__version__ = '0.1.0'

import Globals, Acquisition, AccessControl.Role, OFS.SimpleItem, OFS.PropertyManager
import Products, DateTime
import string
import unittest
import TestResults

_trnr_deprecated_attrs = ()
_trnr_prototype = {
		'title': '', 'suite': '', '_running': None,
	}

class TestRunner (
	OFS.ObjectManager.ObjectManager,
	OFS.SimpleItem.SimpleItem,
	OFS.PropertyManager.PropertyManager,
	):
	"""TestRunner class
	"""

	#################################################
	### Zope class attributes

	name = 'TestRunner'
	id = 'TestRunner'
	meta_type = 'ZUnit TestRunner'
	#icon = 'misc_/ZUnit/TestRunner.gif'

	_properties = (
			{'id': 'title', 'type': 'string', 'mode': 'w'},
			{'id': 'suite', 'type': 'string', 'mode': 'w'},
		)

	manage_options = (
			{'label':'Results', 'action':'manage_main',
			 'help': ('OFSP', 'ObjectManager_Contents.stx')},
			{'label':'Run tests', 'action':'manage_run',
			 'help': ('HiperDom', 'TemplateExpand.stx')},
			{'label':'Properties', 'action':'manage_propertiesForm',
			 'help': ('OFSP', 'Properties.stx')},
			{'label':'Security', 'action':'manage_access',
			 'help': ('OFSP', 'Security.stx')},
			{'label':'Ownership', 'action':'manage_owner',
			 'help': ('OFSP', 'Ownership.stx')},
		)


	__ac_permissions__ = (
			('View management screens',
			 ('manage_tabs',)),
			('Change Permissions',
			 ('manage_access',)),
			('Manage Properties',
			 ('manage_addProperty', 'manage_editProperties',
			 'manage_delProperties', 'manage_changeProperties',)),
			('View',
			 ('', 'index_html', '__call__',)),
			('Run Unit Tests',
			 ('manage_run', 'manage_startTests',)),
			('Know the secrets of life',
			 ('manage_dict',)),
			)


	#################################################
	## Instance constructors

	def __init__ (self, id):
		"Initialize the class instance"
		self.id = id
		for name, value in _trnr_prototype.items ():
			if not hasattr (self, name):
				setattr (self, name, value)

	def __setstate__ (self, state):
		"Initialize when loading from the ZODB"
		Globals.Persistent.__setstate__ (self, state)
		for name, value in _trnr_prototype.items ():
			if not hasattr (self, name):
				setattr (self, name, value)
		for name in _trnr_deprecated_attrs:
			if hasattr (self, name):
				delattr (self, name)


	#################################################
	## Misc ObjectManager stuff
	all_meta_types = ()


	#################################################
	## Management interface
	def manage_startTests (self, REQUEST, **kw):
		"Start the tests and create a TestResults object for it"
		REQUEST.RESPONSE.setHeader ('Content-type', 'text/plain')
		now = self.ZopeTime()
		id = now.strftime ('%Y.%m.%d.%H.%M.%S')
		self.manage_addFolder (id + '.data', 'temporary data holder')
		obj = TestResults.TestResults (id, self.suite, ctx=self)
		if not (REQUEST.has_key ('keep_data') and REQUEST['keep_data']):
			self.manage_delObjects (id + '.data')
		assert obj, "Could not create TestResults!"
		if obj._success:
			obj.title = 'Successful test at %s' % now.ISO()
		else:
			obj.title = 'FAILED test at %s' % now.ISO()
		self._setObject (id, obj)

	manage_run = Globals.HTMLFile ('RunTests', globals())


	#################################################
	## Externally accessible, but not from the manage
	## interface
	
	def index_html (self, REQUEST, **kw):
		"Render the data into a webpage"
		return apply (self.__str__, (self, REQUEST), kw)


	#################################################
	## Externally accessible from Python/DTML only
	
	def __str__ (self, context=None, REQUEST=None, **kw):
		"Render the data"
		return 'Nothing to see here yet, sorry. Move along, move along...'

	def __call__ (self, context=None, REQUEST=None, **kw):
		"Render the data given a context (?)"
		# DocumentTemplate calls us with (None, context)
		if context is None and hasattr (REQUEST, 'has_key') and REQUEST.has_key ('REQUEST'):
			context = REQUEST
			REQUEST = context['REQUEST']
		# FIXME: this will lose our properties when context != self...
		return apply (self.__str__, (context, REQUEST), kw)


	#################################################
	## Methods for internal consumption
	def testInfo (self):
		"Return info on the Suite"
		# I don't know if this is a good sollution...
		return unittest.createTestInstance (self.suite)
		

	#################################################
	## Debugging stuff, in the "what not even God
	## should do" category
	def manage_dict (self, REQUEST=None):
		"Debugging method - meaning is obvious"
		return `self.__dict__`

manage_addTestRunnerForm = Globals.HTMLFile ('TestRunnerAdd', globals ())

def manage_addTestRunner (self, id, REQUEST=None):
	"Add a TestRunner to the current ObjectManager"
	obj = TestRunner (id)
	assert obj, "Could not create TestRunner!"
	if REQUEST:
		if REQUEST.has_key ('title'):
			obj.title = REQUEST['title']
		if REQUEST.has_key ('suite'):
			obj.suite = REQUEST['suite']
	self._setObject (id, obj)
	if REQUEST is not None:
		return self.manage_main (self, REQUEST)
